@@ -1,4 +1,5 @@
-### $Id: minitab.R,v 1.1.1.1 2001/03/23 16:15:26 bates Exp $
+### This file is part of the `foreign' package for R.
+
 ###
 ###             Read stored Minitab worksheets
 ###
@@ -17,9 +18,8 @@
 ### details.
 ###
 ### You should have received a copy of the GNU General Public
-### License along with this program; if not, write to the Free
-### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-### MA 02111-1307, USA
+### License along with this program; if not, a copy is available at
+### http://www.r-project.org/Licenses/
 
 ## Files in the Minitab portable worksheet format represent numbers in a
 ## fixed format (written in Fortran as 5e15.9 or something like that) but
@@ -27,8 +27,8 @@
 ## signif to remove the inaccuracies in the representation in the files.
 
 "read.mtp" <-
-function (fname) {
-  clean <- function(x) if(is.numeric(x)) signif(x, 6) else x
-  val <- .Call("read_mtp", fname, PACKAGE = "foreign")
-  lapply(val, clean)
+function (file) {
+    clean <- function(x) if(is.numeric(x)) signif(x, 6) else x
+    val <- .Call(read_mtp, file)
+    lapply(val, clean)
 }
